==> local-booter-22.2.5-sources.jar: com/xebialabs/deployit/booter/local/AbstractGlobalContextManager.java <==
package com.xebialabs.deployit.booter.local;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Stream;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.xebialabs.deployit.booter.local.utils.Strings;
import com.xebialabs.deployit.plugin.api.reflect.Descriptor;
import com.xebialabs.deployit.plugin.api.reflect.DescriptorRegistry;
import com.xebialabs.deployit.plugin.api.reflect.PropertyDescriptor;
import com.xebialabs.deployit.plugin.api.reflect.Type;

public abstract class AbstractGlobalContextManager implements GlobalContextManager {

    protected static class Pair {
        protected final String key;
        protected final GlobalContext.Value value;

        public Pair(final String key, final GlobalContext.Value value) {
            this.key = key;
            this.value = value;
        }
    }

    protected final Logger logger = LoggerFactory.getLogger(getClass());

    private static final Function<String, GlobalContext.Value> COMPUTE_FUNCTION = s -> new GlobalContext.Value("", null);

    private Map<String, GlobalContext.Value> context;

    public GlobalContextManager withContext(final Map<String, GlobalContext.Value> context) {
        this.context = context;
        return this;
    }

    protected void addContextEntries(Map<String, Object> properties) {
        properties.forEach(this::addContextEntry);
    }

    protected void addContextEntry(Object key, Object value) {
        GlobalContext.Value contextValue = context.computeIfAbsent(key.toString(), COMPUTE_FUNCTION);
        context.put(key.toString(), new GlobalContext.ExplicitValue(contextValue.description, value.toString()));
    }

    protected Stream<Pair> getContextEntries() {
        List<String> keys = new ArrayList<>(context.keySet());
        return keys.stream()
                .sorted()
                .map(key -> new Pair(key, context.computeIfAbsent(key, COMPUTE_FUNCTION)));
    }

    public void validateValues(File configFile) {
        List<String> messages = validateContextEntries();
        if (!messages.isEmpty()) {
            String join = messages.stream().reduce((s, t) -> s + "\n" + t).get();
            logger.error(join);
            String errorMessage = "Could not initialize the default values, please look at the log and correct the configuration file " + "\n" + join;
            if (configFile != null) {
                errorMessage = "Could not initialize the default values, please look at the log and correct the defaults file: " + configFile + "\n" + join;
            }
            throw new IllegalStateException(errorMessage);
        }
    }

    protected List<String> validateContextEntries() {
        List<String> messages = new ArrayList<>();
        for (String prop : context.keySet()) {
            int i = prop.lastIndexOf('.');
            String type = prop.substring(0, i);
            String propertyName = prop.substring(i + 1);

            Type tType = Type.valueOf(type);
            if (LocalDescriptorRegistry.exists(tType)) {
                Descriptor descriptor = DescriptorRegistry.getDescriptor(tType);
                if (descriptor.isVirtual()) continue;
                PropertyDescriptor propertyDescriptor = descriptor.getPropertyDescriptor(propertyName);
                if (propertyDescriptor != null) {
                    try {
                        propertyDescriptor.getDefaultValue();
                        if (propertyDescriptor.isRequired() && propertyDescriptor.isHidden()
                                && Strings.isEmpty(GlobalContext.lookup(propertyDescriptor))) {
                            messages.add(String.format("Cannot register empty default value for hidden required property [%s]", prop));
                        }
                    } catch (RuntimeException re) {
                        messages.add(String.format("Incorrect default registered: [%s] with value %s",
                                prop, context.computeIfAbsent(prop, COMPUTE_FUNCTION).getValue()));
                    }
                }
            }
        }
        return messages;
    }

    protected String header() {
        return "# Note: If you modify this file, you must restart the XL Deploy server.";
    }

    protected String toComment(GlobalContext.Value value) {
        StringBuilder sb = new StringBuilder(value.description);
        if (value instanceof GlobalContext.InheritedValue) {
            GlobalContext.InheritedValue inheritedValue = (GlobalContext.InheritedValue) value;
            sb
                    .append(" (inherited from: ")
                    .append(inheritedValue.getSuperTypeProperty())
                    .append(")");
        }
        return sb.toString();
    }
}

==> local-booter-23.1.0-331.1639-sources.jar: com/xebialabs/deployit/booter/local/AbstractGlobalContextManager.java <==
package com.xebialabs.deployit.booter.local;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Stream;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.xebialabs.deployit.booter.local.utils.Strings;
import com.xebialabs.deployit.plugin.api.reflect.Descriptor;
import com.xebialabs.deployit.plugin.api.reflect.DescriptorRegistry;
import com.xebialabs.deployit.plugin.api.reflect.PropertyDescriptor;
import com.xebialabs.deployit.plugin.api.reflect.Type;

public abstract class AbstractGlobalContextManager implements GlobalContextManager {

    protected static class Pair {
        protected final String key;
        protected final GlobalContext.Value value;

        public Pair(final String key, final GlobalContext.Value value) {
            this.key = key;
            this.value = value;
        }
    }

    protected final Logger logger = LoggerFactory.getLogger(getClass());

    private static final Function<String, GlobalContext.Value> COMPUTE_FUNCTION = s -> new GlobalContext.Value("", null);

    private Map<String, GlobalContext.Value> context;

    public GlobalContextManager withContext(final Map<String, GlobalContext.Value> context) {
        this.context = context;
        return this;
    }

    protected void addContextEntries(Map<String, Object> properties) {
        properties.forEach(this::addContextEntry);
    }

    protected void addContextEntry(Object key, Object value) {
        GlobalContext.Value contextValue = context.computeIfAbsent(key.toString(), COMPUTE_FUNCTION);
        context.put(key.toString(), new GlobalContext.ExplicitValue(contextValue.description, value.toString()));
    }

    protected Stream<Pair> getContextEntries() {
        List<String> keys = new ArrayList<>(context.keySet());
        return keys.stream()
                .sorted()
                .map(key -> new Pair(key, context.computeIfAbsent(key, COMPUTE_FUNCTION)));
    }

    public void validateValues(File configFile) {
        List<String> messages = validateContextEntries();
        if (!messages.isEmpty()) {
            String join = messages.stream().reduce((s, t) -> s + "\n" + t).get();
            logger.error(join);
            String errorMessage = "Could not initialize the default values, please look at the log and correct the configuration file " + "\n" + join;
            if (configFile != null) {
                errorMessage = "Could not initialize the default values, please look at the log and correct the defaults file: " + configFile + "\n" + join;
            }
            throw new IllegalStateException(errorMessage);
        }
    }

    protected List<String> validateContextEntries() {
        List<String> messages = new ArrayList<>();
        for (String prop : context.keySet()) {
            int i = prop.lastIndexOf('.');
            String type = prop.substring(0, i);
            String propertyName = prop.substring(i + 1);

            // TODO S-88045 context should be associated with the type
            Type tType = Type.valueOf(type);
            if (DescriptorRegistry.exists(tType)) {
                Descriptor descriptor = DescriptorRegistry.getDescriptor(tType);
                if (descriptor.isVirtual()) continue;
                PropertyDescriptor propertyDescriptor = descriptor.getPropertyDescriptor(propertyName);
                if (propertyDescriptor != null) {
                    try {
                        propertyDescriptor.getDefaultValue();
                        if (propertyDescriptor.isRequired() && propertyDescriptor.isHidden()
                                && Strings.isEmpty(GlobalContext.lookup(propertyDescriptor))) {
                            messages.add(String.format("Cannot register empty default value for hidden required property [%s]", prop));
                        }
                    } catch (RuntimeException re) {
                        messages.add(String.format("Incorrect default registered: [%s] with value %s",
                                prop, context.computeIfAbsent(prop, COMPUTE_FUNCTION).getValue()));
                    }
                }
            }
        }
        return messages;
    }

    protected String header() {
        return "# Note: If you modify this file, you must restart the XL Deploy server.";
    }

    protected String toComment(GlobalContext.Value value) {
        StringBuilder sb = new StringBuilder(value.description != null ? value.description.replace("\n", " ") : value.description);
        if (value instanceof GlobalContext.InheritedValue) {
            GlobalContext.InheritedValue inheritedValue = (GlobalContext.InheritedValue) value;
            sb
                    .append(" (inherited from: ")
                    .append(inheritedValue.getSuperTypeProperty())
                    .append(")");
        }
        return sb.toString();
    }
}
